# Joh, Yong-iL <tolkien@nownuri.net>
#
# This gets the latest DaemonNews entries.

package DaemonNews::Plugin;

use Perlbot;

use POSIX;
use HTTP::Lite;
require HTML::TokeParser;

sub get_hooks {
  return { public => \&on_public, msg => \&on_msg };
}

sub on_public {
  my $conn = shift;
  my $event = shift;
  my $args;

  ($args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

  if($args =~ /^-daemonnews\s*/  || $args =~ /^-dn\s*/) {
      get_dn($conn, $event, $event->{to}[0]);
  }
}

sub on_msg {
  my $conn = shift;
  my $event = shift;
  my $args;

  ($args = $event->{args}[0]) =~ tr/[A-Z]/[a-z]/;

  if($args =~ /^-daemonnews\s*/  || $args =~ /^-dn\s*/) {
      get_dn($conn, $event, $event->nick);
  }
}

sub get_dn {
  my $conn = shift;
  my $event = shift;
  my $who = shift;

  $args = $event->{args}[0];
  my($term, $min, $max) = split(/ /, $args, 3);

  $term =~ s/^-daemonnews\s*//;
  $term =~ s/^-dn\s*//;
  $term =~ s/\s+/+/g;

  $min =~ s/.*(\d+).*/\1/;
  $max =~ s/.*(\d+).*/\1/;

  if ($max eq '' || $max < 1) { $max = 5; }
  if ($min eq '' || $min < 1) { $min = 0;
  } else { $min -= 1; }

  if(!defined($pid = fork)) {
    $conn->privmsg($who, "error in DaemonNews plugin...");
    return;
  }

  if($pid) {
    #parent

    $SIG{CHLD} = sub { wait; };
    return;

  } else {
    # child

    my ($url, $file, $http, $stat);
    $url = "http://daily.daemonnews.org/";
    $file = "${datadir}/daemonnews.html";

    $http = new HTTP::Lite;
    $stat = $http->request($url);
    if ($stat == 200) {
	open FILE, ">" . $file;
	print FILE $http->body();
	close FILE;
    }

    open FILE, $file;

    my $p = HTML::TokeParser->new(*FILE) || die "Can't open: $!";

    $conn->privmsg($who, "Daemon News:");

    my (@tmp, $i, $lcnt, $dummy);
    $dummy = $min + $max - 1;

    $lcnt = 0;
    while (my $token = $p->get_token) {
      if ($token->[0] eq 'C') {
	if ($token->[1] =~ /START/) {
	  $lcnt++;
	  $token = $p->get_tag("table");
	  $token = $p->get_tag("table");
	  $token = $p->get_tag("font");
	  my $title = $p->get_trimmed_text("/font");
	  $token = $p->get_tag("a");
	  my $url = $token->[1]{href} || "-";
	  $token = $p->get_tag("td");
	  $token = $p->get_tag("font");
	  my $date = $p->get_trimmed_text("/font");

	  @tmp = split(/ /, $date);
	  $tmp[1] = substr($tmp[1], 0, 3);
	  $tmp[2] = substr($tmp[2], 2, 2);

	  $conn->privmsg($who, "\[$tmp[1]/$tmp[0]/$tmp[2]\] $title\n");
	  if ($url ne "-") {
	    $conn->privmsg($who, "            $url\n");
	  }
	}
      } else { next; }
      if ($lcnt > $dummy) { last; }
    }
    close FILE;
    $conn->{_connected} = 0;
    exit 0;
  }
}

1;
